import micromatch from 'micromatch';
import { initIssues, initCounters } from './issues/initializers.js';
import { relative } from './util/path.js';
const hasHint = (hints, hint) => Array.from(hints).some(item => item.identifier === hint.identifier && item.type === hint.type && item.workspaceName === hint.workspaceName);
export class IssueCollector {
    cwd;
    rules;
    filters;
    issues = initIssues();
    counters = initCounters();
    referencedFiles = new Set();
    configurationHints = new Set();
    ignorePatterns = new Set();
    isMatch;
    constructor({ cwd, rules, filters }) {
        this.cwd = cwd;
        this.rules = rules;
        this.filters = filters;
        this.isMatch = () => false;
    }
    addIgnorePatterns(patterns) {
        patterns.forEach(pattern => this.ignorePatterns.add(pattern));
        const p = [...this.ignorePatterns];
        this.isMatch = (filePath) => micromatch.isMatch(filePath, p, { dot: true });
    }
    addFileCounts({ processed, unused }) {
        this.counters.processed += processed;
        this.counters.total += processed + unused;
    }
    addFilesIssues(filePaths) {
        filePaths.forEach(filePath => {
            if (this.filters.dir && !filePath.startsWith(this.filters.dir + '/'))
                return;
            if (this.referencedFiles.has(filePath))
                return;
            if (this.isMatch(filePath))
                return;
            this.issues.files.add(filePath);
            this.counters.files++;
            this.counters.processed++;
        });
    }
    addIssue(issue) {
        if (this.filters.dir && !issue.filePath.startsWith(this.filters.dir + '/'))
            return;
        if (this.isMatch(issue.filePath))
            return;
        const key = relative(this.cwd, issue.filePath);
        issue.severity = this.rules[issue.type];
        this.issues[issue.type][key] = this.issues[issue.type][key] ?? {};
        if (!this.issues[issue.type][key][issue.symbol]) {
            this.issues[issue.type][key][issue.symbol] = issue;
            this.counters[issue.type]++;
        }
    }
    addConfigurationHint(issue) {
        if (!hasHint(this.configurationHints, issue)) {
            this.configurationHints.add(issue);
        }
    }
    getIssues() {
        return {
            issues: this.issues,
            counters: this.counters,
            configurationHints: this.configurationHints,
        };
    }
}
